import torch
import torch.nn as nn
import numpy as np
import pandas as pd
import sys


class LinearRegressionModel(nn.Module):
    def __init__(self, input_dim, output_dim):
        super(LinearRegressionModel, self).__init__()
        self.linear = nn.Linear(input_dim, output_dim)

    def forward(self, x):
        out = self.linear(x)
        return out


def read_data_file(filepath, sizes_index, prices_index):
    df = pd.read_csv(filepath, sep='\t', header=None, index_col=None)
    sizes = df[sizes_index].tolist() if sizes_index is not None else None
    prices = df[prices_index].tolist() if prices_index is not None else None
    return sizes, prices


def predict_prices(model):
    x, _ = read_data_file('test-A/in.tsv', 7, None)
    data_array = np.array(x, dtype=np.float32)
    data_array = data_array.reshape(-1, 1)
    return model(torch.from_numpy(data_array).requires_grad_())


def setup_model(device):
    model = LinearRegressionModel(1,1)
    model.to(device)
    return model


def train_model(model, device, epochs):
    criterion = nn.MSELoss()

    learning_rate = 0.00001
    optimizer = torch.optim.SGD(model.parameters(), lr=learning_rate)

    x,y = read_data_file("train/train.tsv", 8, 0)

    x = np.array(x)
    y = np.array(y)
    x = x.reshape(-1, 1)
    y = y.reshape(-1, 1)

    for epoch in range(epochs):
        epoch += 1
        inputs = torch.FloatTensor(x)
        labels = torch.FloatTensor(y)

        optimizer.zero_grad()

        outputs = model(inputs)

        loss = criterion(outputs, labels)

        loss.backward()

        optimizer.step()


def save_predictions(prediction_model):
    predictions = prediction_model.data.numpy()
    with open('test-A/out.tsv', 'w') as outfile:
        for prediction in predictions:
            outfile.write(str(prediction[0])+'\n')


def main():
    epochs = 1000
    device = torch.device("cpu")

    model = setup_model(device)
    train_model(model, device, epochs)

    prediction_model = predict_prices(model)
    save_predictions(prediction_model)


if __name__ == '__main__':
    main()
